

--[[
TODO:
replace the word matching with something more efficient

write to framebuffer when it is not the focus

add a cursor to text
make text selectable
colour strings

delete/hide project

--]]

local tinsert = table.insert

UIObject = {} --basic methods common to all objects

UIObject.__index = UIObject


function UIObject.Create(parent)
	if parent.children then
	local temp = {}
	temp.__index = UIObject
	setmetatable(temp, temp)
	temp.x = 0
	temp.y = 0
	temp.rot = 0
	temp.width = 0
	temp.height = 0
	temp.minheight = 0
	temp.minwidth = 0
	temp.children = {}
	temp.parent = parent
	temp.shown = true
	temp.isshown = true
	temp.scalex = 1
	temp.scaley = 1
	
	tinsert(Frames, temp)
	tinsert(parent.children, temp)
	return temp
	end
end

function UIObject.SetScale(self,x,y)
	if not y then
		y = x
	end
	
	self.scalex = x
	self.scaley = y
end

function UIObject.GetScale(self)
	return self.scalex, self.scaley
end

function UIObject.SetMinWidth(self, w)
	self.minwidth = w
end

function UIObject.GetPoint(self)
	return self.x, self.y
end

function UIObject.SetScaleX(self,x)
	self.scalex = x
end

function UIObject.IsVisible(self)
	return self.parent:IsVisible() and self.isshown
end

function UIObject.GetScaleX(self)
	return self.scalex * self.parent:GetScaleX()
end

function UIObject.SetScaleY(self,y)
	
	self.scaley = y
end

function UIObject.SetParent(self, parent)
	for i = 1, #self.parent.children do
		if self.parent.children[i] == self then
			tremove(self.parent.children, i)
		end
	end
	tinsert(parent.children, self)
	self.parent = parent
end

function UIObject.GetScaleY(self)
	return self.scaley * self.parent:GetScaleY()
end

function UIObject.SetWidth(self, width)
	self.width = math.max(width, self.minwidth)
end

function UIObject.SetHeight(self, height)
	self.height = math.max(height, self.minheight)
end



function UIObject.SetSize(self,x,y)
	self.width = math.max(x, self.minwidth)
	self.height = math.max(y, self.minheight)
end

function UIObject.GetX(self)
	--if self.backdrop then
	--	return self.x + self.parent:GetX() + self.bdindent*self:GetScaleX()
	--else
		return self.x*self:GetScaleX() + self.parent:GetX()
	--end
end

function UIObject.GetY(self)
--	if self.backdrop then
	--	return self.y + self.parent:GetY() + self.bdindent*self:GetScaleY()
	--else
		return self.y*self:GetScaleY() + self.parent:GetY()
	--end
end

function UIObject.GetWidth(self)
	return self.width  * self.parent:GetScaleX()
end
function UIObject.GetHeight(self)
	return self.height * self.parent:GetScaleY()
end

function UIObject.GetRot(self)
	return self.rot + self.parent:GetRot()
end

function UIObject.SetRot(self,rot)
	return self.rot + rot
end

function UIObject.SetPoint(self, x, y)
	self.x = x
	self.y = y
end

function UIObject.Show(self)
	self.isshown = true
end

function UIObject.Hide(self)
	self.isshown = nil
end

local Texture = {}
Texture.__index = UIObject
setmetatable(Texture, Texture)

function Texture.Create(parent)
	local temp = UIObject.Create(parent)
		temp.__index = Texture
		setmetatable(temp, temp)
		
		
		return temp
end

function Texture.SetTexture(self, path) -- textures stored as quads to handle settecoords
	self.image = love.graphics.newImage(path)
	self.basewidth = self.image:getWidth()
	self.baseheight = self.image:getHeight()
	self.quadheight = 1
	self.quadwidth = 1
	self.quad = love.graphics.newQuad(0,0,self.basewidth,self.baseheight,self.basewidth,self.baseheight)
	self:SetTexCoord(0,1,0,1)
end

function Texture.SetTexCoord(self, left, right, top, bottom)
	self.quadcoords = {left, right, top, bottom}
	left = self.basewidth*left
	right = self.basewidth*right
	top = self.baseheight*top
	bottom = self.baseheight*bottom
	local width = right - left
	local height = bottom-top
	self.quadheight = height
	self.quadwidth = width
	self.quad = love.graphics.newQuad(left, top, width, height, self.basewidth, self.baseheight)
	
end

function Texture.SetColor(self, color)
	self.color = color
end

function Texture.draw(self)
	if self.color then
			love.graphics.setColorMode("modulate")
			love.graphics.setColor(unpack(self.color))
	else
		love.graphics.setColorMode("replace")
		love.graphics.setColor(255,255,255,255)
	end
	
	love.graphics.drawq(self.image, self.quad, self:GetX(), self:GetY(), self:GetRot(), (self.width/self.quadwidth),(self.height/self.quadheight))
	love.graphics.setColorMode("modulate")
	love.graphics.setColor(255,255,255,255)
end

local Frame = {} --frame objects can handle mouse and keyboard events
Frame.__index = UIObject
setmetatable(Frame, Frame)

function Frame.Create(parent)
	local temp = UIObject.Create(parent)
		temp.__index = Frame
		setmetatable(temp, temp)
		temp.children = {}
		temp.scripts = {}
		temp.mouseEnabled = true
		
		return temp
end

function Frame.CreateTexture(self)
	return Texture.Create(self)
end

function Frame.StartSizing(self)
	local mx, my = love.mouse.getPosition()
	self.xoff = mx - self.x
	self.yoff = my - self.y
	self.sizing = true
end

function Frame.StartMoving(self)
	local mx, my = love.mouse.getPosition()
	self.xoff = mx - self.x
	self.yoff = my - self.y
	--self.oldx, self.oldy = self:GetPoint()
	self.moving = true
end

function Frame.StopMovingOrSizing(self)
	self.moving = false
	self.sizing = false 
end

function Frame.MouseDown(self,x,y,button)
	local found
	if self:IsVisible() then
	for _,child in pairs(self.children) do
		if child.mouseover then
			found = true
		end
	end
	if not found then
		if self.scripts.OnMouseDown then
			self:SetFocus(true) 
			for _,f in pairs(Frames) do
				if not(f == self) then
					if f.SetFocus then
						f:SetFocus(nil)
					end
				end 
			end 
			self.scripts.OnMouseDown(self,x,y,button)
		end
	end
	end
end



function Frame.SetFocus(self,var)
	if self.focus then
		if not var then
			self.focus = var
			self.bx, self.by = self:GetX(), self:GetY()
			--self.buffer = love.graphics.newFramebuffer()
			--love.graphics.setRenderTarget(self.buffer)
			--for _,child in pairs(self.children) do
			--	if child.isshown then
			--		child:draw()
			--	end
			--end

			--love.graphics.setRenderTarget()
		end
	else
		self.focus = var
	end
end

function Frame.MouseUp(self,x,y,button)
	local found
	for _,child in pairs(self.children) do
		if child.mouseover then
			found = true
		end
	end
	if not found then
		--do this frames mousedown stuff
		if self.scripts.OnMouseUp then
			self.scripts.OnMouseUp(self, x, y, button)
		end
	end
end

function Frame.CheckMouse(self,x,y)
	if x > self:GetX() and x <self:GetX()+self:GetWidth() and y > self:GetY() and y < self:GetY()+self:GetHeight() then
		if not self.mouseover then
			self.mouseover = true
			if self.scripts.OnEnter then
				self.scripts.OnEnter()
			end
		end
	else
		if self.mouseover then
			self.mouseover = nil
			if self.scripts.OnLeave then
				self.scripts.OnLeave()
			end
		end
	end
end


function Frame.SetScript(self, script, func)
	Scripts[script][self] = func
	self.scripts[script] = func
end

function Frame.draw(self)
	--if self.focus or not self.buffer then
		for _,child in pairs(self.children) do
			if child.isshown then
				child:draw()
			end
		end
	--else
	--	love.graphics.setColor(255,255,255,255)
		
		--love.graphics.draw(self.buffer, self:GetX()-self.bx, self:GetY()-self.by, self:GetRot(), self:GetScaleX(), self:GetScaleY())
	

	--end
end

function Frame.UpdateBackdrop(self)
	local bd = self.backdrop
	local indent = self.bdindent
	
	local sx = self:GetScaleX()
	local sy = self:GetScaleY()
	
	bd.TopLeft:SetWidth(indent*sx)
	bd.TopLeft:SetHeight(indent*sy)
	bd.TopLeft:SetTexCoord(0,1/(bd.TopLeft.basewidth/indent), 0,(1/(bd.TopLeft.basewidth/indent)))
	
	bd.Top:SetWidth((self.width - 2*indent)*sx)
	bd.Top:SetHeight(indent*sy)
	bd.Top:SetPoint(indent,0)
	bd.Top:SetTexCoord((1/(bd.Top.basewidth/indent)), 1-(1/(bd.Top.basewidth/indent)), 0,(1/(bd.Top.basewidth/indent)))
	
	bd.TopRight:SetWidth(indent*sx)
	bd.TopRight:SetHeight(indent*sx)
	bd.TopRight:SetPoint((self.width-indent),0)
	bd.TopRight:SetTexCoord(1-(1/(bd.TopRight.basewidth/indent)), 1, 0, (1/(bd.TopRight.basewidth/indent)))
	
	bd.Left:SetWidth(indent*sx)
	bd.Left:SetHeight((self.height - 2*indent)*sy)
	bd.Left:SetPoint(0,indent)
	bd.Left:SetTexCoord(0,(1/(bd.Left.basewidth/indent)), (1/(bd.Left.basewidth/indent)),1-(1/(bd.Left.basewidth/indent)))
	
	bd.Center:SetWidth((self.width - 2*indent)*sx)
	bd.Center:SetHeight((self.height - 2*indent)*sy)
	bd.Center:SetPoint(indent,indent)
	bd.Center:SetTexCoord((1/(bd.Center.basewidth/indent)), 1-(1/(bd.Center.basewidth/indent)), (1/(bd.Center.basewidth/indent)), 1-(1/(bd.Center.basewidth/indent)))
	
	bd.Right:SetWidth(indent*sx)
	bd.Right:SetHeight((self.height- 2*indent)*sy)
	bd.Right:SetPoint((self.width-indent),indent)
	bd.Right:SetTexCoord(1-(1/(bd.Right.basewidth/indent)), 1, (1/(bd.Right.basewidth/indent)), 1-(1/(bd.Right.basewidth/indent)))
	
	bd.BottomLeft:SetWidth(indent*sx)
	bd.BottomLeft:SetHeight(indent*sy)
	bd.BottomLeft:SetPoint(0, (self.height-indent))
	bd.BottomLeft:SetTexCoord(0, (1/(bd.BottomLeft.basewidth/indent)), 1-(1/(bd.BottomLeft.basewidth/indent)), 1)
	
	bd.Bottom:SetWidth((self.width - 2*indent)*sx)
	bd.Bottom:SetHeight(indent*sy)
	bd.Bottom:SetPoint(indent, (self.height-indent))
	bd.Bottom:SetTexCoord((1/(bd.Bottom.basewidth/indent)), 1-(1/(bd.Bottom.basewidth/indent)), 1-(1/(bd.Bottom.baseheight/indent)), 1)
	
	bd.BottomRight:SetWidth(indent*sx)
	bd.BottomRight:SetHeight(indent*sy)
	bd.BottomRight:SetPoint((self.width-indent), (self.height-indent))
	bd.BottomRight:SetTexCoord(1-(1/(bd.BottomRight.basewidth/indent)), 1, 1-(1/(bd.BottomRight.basewidth/indent)), 1)

	
end

function Frame.SetBackdrop(self, path, indent)
	self.minwidth = 2*indent
	self.minheight = 2*indent
	self.backdrop = {}
	local bd = self.backdrop
	self.bdindent = indent
	bd.TopLeft = self:CreateTexture()
	bd.TopLeft:SetWidth(indent)
	bd.TopLeft:SetHeight(indent)
	bd.TopLeft:SetPoint(0,0)
	bd.TopLeft:SetTexture(path)
	bd.TopLeft:SetTexCoord(0,1/(bd.TopLeft.basewidth/indent), 0,(1/(bd.TopLeft.basewidth/indent)))
	
	bd.Top = self:CreateTexture()
	bd.Top:SetWidth(self.width - 2*indent)
	bd.Top:SetHeight(indent)
	bd.Top:SetPoint(indent,0)
	bd.Top:SetTexture(path)
	bd.Top:SetTexCoord((1/(bd.Top.basewidth/indent)), 1-(1/(bd.Top.basewidth/indent)), 0,(1/(bd.Top.basewidth/indent)))
	
	bd.TopRight = self:CreateTexture()
	bd.TopRight:SetWidth(indent)
	bd.TopRight:SetHeight(indent)
	bd.TopRight:SetPoint(self.width-indent,0)
	bd.TopRight:SetTexture(path)
	bd.TopRight:SetTexCoord(1-(1/(bd.TopRight.basewidth/indent)), 1, 0, (1/(bd.TopRight.basewidth/indent)))
		
	bd.Left = self:CreateTexture()
	bd.Left:SetWidth(indent)
	bd.Left:SetHeight(self.height - 2*indent)
	bd.Left:SetPoint(0,indent)
	bd.Left:SetTexture(path)
	bd.Left:SetTexCoord(0,(1/(bd.Left.basewidth/indent)), (1/(bd.Left.basewidth/indent)),1-(1/(bd.Left.basewidth/indent)))
		
	bd.Center = self:CreateTexture()
	bd.Center:SetWidth(self.width - 2*indent)
	bd.Center:SetHeight(self.height - 2*indent)
	bd.Center:SetPoint(indent,indent)
	bd.Center:SetTexture(path)
	bd.Center:SetTexCoord((1/(bd.Center.basewidth/indent)), 1-(1/(bd.Center.basewidth/indent)), (1/(bd.Center.basewidth/indent)), 1-(1/(bd.Center.basewidth/indent)))

	bd.Right = self:CreateTexture()
	bd.Right:SetWidth(indent)
	bd.Right:SetHeight(self.height- 2*indent)
	bd.Right:SetPoint(self.width-indent,indent)
	bd.Right:SetTexture(path)
	bd.Right:SetTexCoord(1-(1/(bd.Right.basewidth/indent)), 1, (1/(bd.Right.basewidth/indent)), 1-(1/(bd.Right.basewidth/indent)))
	
	bd.BottomLeft = self:CreateTexture()
	bd.BottomLeft:SetWidth(indent)
	bd.BottomLeft:SetHeight(indent)
	bd.BottomLeft:SetPoint(0, self.height-indent)
	bd.BottomLeft:SetTexture(path)
	bd.BottomLeft:SetTexCoord(0, (1/(bd.BottomLeft.basewidth/indent)), 1-(1/(bd.BottomLeft.basewidth/indent)), 1)
	
	bd.Bottom = self:CreateTexture()
	bd.Bottom:SetWidth(self.width - 2*indent)
	bd.Bottom:SetHeight(indent)
	bd.Bottom:SetPoint(indent, self.height-indent)
	bd.Bottom:SetTexture(path)
	bd.Bottom:SetTexCoord((1/(bd.Bottom.basewidth/indent)), 1-(1/(bd.Bottom.basewidth/indent)), 1-(1/(bd.Bottom.baseheight/indent)), 1)
		
	bd.BottomRight = self:CreateTexture()
	bd.BottomRight:SetWidth(indent)
	bd.BottomRight:SetHeight(indent)
	bd.BottomRight:SetPoint(self.width-indent, self.height-indent)
	bd.BottomRight:SetTexture(path)
	bd.BottomRight:SetTexCoord(1-(1/(bd.BottomRight.basewidth/indent)), 1, 1-(1/(bd.BottomRight.basewidth/indent)), 1)

end



local FontString = {}
FontString.__index = UIObject
setmetatable(FontString, FontString)

function FontString.Create(parent, font, size)

	local temp = UIObject.Create(parent)
	temp.__index = FontString
	setmetatable(temp, temp)
	
	temp.text = ""
	if size then
		temp.font = love.graphics.newFont(font,size)
	else
		temp.font = love.graphics.newFont(font)
	end

	temp.words = {}
	return temp
end

function FontString.GetTextWidth(self)
	return self.font:getWidth(self.text)
end

function words(s)
 local r = {};
 local last = ""
	for p = 1, s:len() do
		local c = s:sub(p, p);
		if c:find("%w") then
			if last:find("%w") then
				r[#r].w = r[#r].w .. c
			else
				r[#r+1] = {w = c}
			end
			--elseif  (c == " ") or (c == "\n") or (c == "\t") then
			--table.insert(r, {w = c})
		else
			table.insert(r, {w = c})
		end
		last = c
	end
	
	return r
end



function FontString.SetColor(self, color)
	self.color = color
end

function FontString.Format(self)


	self.words = words(self.text)
	local x,y = 0,0
	local ssx, ssy, esx, esy = self.parent.ssx, self.parent.ssy, self.parent.esx, self.parent.esy --start of selection, end of selection
	for _,w in pairs(self.words) do
		if w.w:find("%w+") then
			if not Strings[w.w] then
				Strings[w.w] = {}
			end
			Strings[w.w][w] = w
		end
			
			w.y = y
			w.x = x
			w.parent = self
			w.wid = self.font:getWidth(w.w)
			w.hi = self.font:getHeight(w.w)
			w.sel = nil
		if esx then
			local lx, ty, rx, by = w.x, w.y, w.x+w.wid, w.y+w.hi--center x, center y of letter
			if (lx >= ssx or ty >ssy+w.hi/2) and (rx <= esx or by < esy-w.hi/2 ) then
				if by >= ssy and ty <= esy then
					
					w.sel = true
				end
			end
			--[[
			if ((w.x + w.wid/2) >= ssx and (w.x + w.wid/2) <= self.width) or ((w.x + w.wid/2) <= ssx and (w.x + w.wid/2) >= 0 ) then
				if (w.y + w.hi/2) >= ssy and (w.y + w.hi/2) <= esy then
					w.sel = true
				end
			end
			--]]
		end
			
		if w.w == "\n" then
			y = y + self.font:getHeight()*1.2
			x = 0
		elseif w.w == "\t" then
			x = x + self.font:getWidth("    ")
		else
			if StringTypes[w.w] then
				w.color = StringTypes[w.w]
			end 
			x = x + self.font:getWidth(w.w)
		end
	end
end

function FontString.SetText(self, text)
	self.text = text or ""
end

function FontString.GetText(self)
	return self.text
end


function FontString.draw(self)
	love.graphics.setFont(self.font)
	if self.format then
		for _,w in pairs(self.words) do
			if w.sel then
				love.graphics.setColor(255,255,255,120)
				love.graphics.rectangle("fill", self:GetX()+w.x*self:GetScaleX(), self:GetY()+w.y*self:GetScaleY(), w.wid,w.hi)
			end		
		
			if w.color then
				love.graphics.setColor(unpack(w.color))
			else
				love.graphics.setColor(255,255,255,255)
			end
			if self.color then --override any color formatting if needed
				love.graphics.setColor(unpack(self.color))
			end	

			love.graphics.print(w.w, self:GetX()+w.x*self:GetScaleX(), self:GetY()+w.y*self:GetScaleY(), self:GetRot(), self:GetScaleX(), self:GetScaleY())
		end
	else
		if self.color then --override any color formatting if needed
			love.graphics.setColor(unpack(self.color))
		end	
		love.graphics.print(self.text, self:GetX(), self:GetY(), self:GetRot(), self:GetScaleX(), self:GetScaleY())
	end
end

function Frame.CreateFontString(self, path, size)
	return FontString.Create(self,path, size)
end


--=================== EditBox ===============================
--===========================================================

local EditBox = {}
EditBox.__index = Frame
setmetatable(EditBox, EditBox)


function EditBox.Create(parent, font, size)

	local temp = CreateFrame("Frame", parent)
	temp.__index = EditBox
	setmetatable(temp, temp)

	temp.body = temp:CreateFontString(font,size)
	temp.body:SetPoint(0,0)
	
	temp.ssx = 0 --start select x
	temp.ssy = 0 --
	
	temp.esx = 0 --end select x
	temp.esy = 0
	
temp:SetScript("OnKeyDown", function(self, key, uni)
		
																if self.focus then 
																	self:EditText(key,uni)													
																else
																	--self:SetText(self:GetText())
																end
															end)	
	
	temp:SetScript("OnEnter", function(self) end)
	temp:SetScript("OnMouseDown", function(self, x, y)
																	
																	self.ssx = x - self:GetX()
																	self.ssy = y - self:GetY()
																end)
	temp:SetScript("OnMouseUp", function(self, x, y)
																self.esx = x - self:GetX()
																self.esy = y - self:GetY()
																if self.body then
																	self.body:Format()
																end
															end)
	return temp
end

function EditBox.SetFocus(self,var)
	if self.focus then
		if not var then
			self.focus = var
			
			self.buffer = love.graphics.newFramebuffer(self:GetWidth(), self:GetHeight())
			function self.GetX()
				return 0
			end
			function self.GetY()
				return 0
				end
			love.graphics.setRenderTarget(self.buffer)
			for _,child in pairs(self.children) do
				if child.isshown then
					child:draw()
				end
			end

			love.graphics.setRenderTarget()
			self.GetX = nil
			self.GetY = nil
		end
	else
		self.focus = var
	end
end

function EditBox.draw(self)
	if self.focus or not self.buffer then
		for _,child in pairs(self.children) do
			if child.isshown then
				child:draw()
			end
		end
	else
		love.graphics.setColor(255,255,255,255)
		
	love.graphics.draw(self.buffer, self:GetX(), self:GetY(), self:GetRot(), self:GetScaleX(), self:GetScaleY())
	end
end


function EditBox.SetText(self, text)
	self.body:SetText(text)
	if self.format then --format if its needed
		self.body:Format()
	else
		
	end
	
	if self.wrapparent and self.isshown then --set the parents size if needed
		self.body.width = 0
		self.body.height = 0
	
		for line in self.body.text:gmatch("([^\n]*)\n?") do -- match per line to get max height and width
			if self.body.font:getWidth(line) > self.body.width then
				self.body.width = self.body.font:getWidth(line)
			end
			self.body.height = self.body.height + self.body.font:getHeight()*1.2
		end
		if self.parent.subframes then
			for f in pairs(self.parent.subframes) do
				f.dx = f.x - self.parent:GetWidth()
				f.dy = f.y - self.parent:GetHeight()
			end
		end
		
		self.parent:SetWidth(self.body.width+50)
		self.parent:SetHeight(self.body.height+60)
		self:SetWidth(self.body.width)
		self:SetHeight(self.body.height)
		if self.parent.backdrop then
			self.parent:UpdateBackdrop()
		end
			
		if self.parent.subframes then
			for f in pairs(self.parent.subframes) do
				f.x = self.parent:GetWidth() + f.dx
				f.y = self.parent:GetHeight() + f.dy
			end
		end
	end
end

function EditBox.EditText(self,key,uni)
	local old = self:GetText()
		
	if uni == 8 then --backspace
		old = old:sub(1, old:len()-1)
	elseif uni == 13 then --return
		old = old.."\n"
	elseif key == "shift" or key == "ctrl" or key == "alt" then
	elseif uni == 9 then
		old = old.."   "
	else
		if uni > 0 then
			local s = string.char(uni)
			if s then
				old = old..s
			end
		end
	end
	
	---[[ this will change the name of the box
	if not self.parent.file then
		if old:find("function") then
			local name = old:match("function%s+%w+%.(%w+)")
			name = name or old:match("function%s+(%w+)")
			name = name or old:match("(%w+)%s-=%s-function")
			if name then
				self.parent:SetTitle(name)
			end
		else
			self.parent:SetTitle("Variables")
		end
	end
	--]]
	
	if old:find("\n.+function.+") then
		local body, extra = old:match("(.+)\n(.-function.+)")
			local t = CreateFrame("EditFrame", self.parent)
			t:SetPoint(0,self.parent:GetHeight())
			t.text:SetText(extra)
			t:SetWidth(t.text:GetWidth()+50)
			t:SetHeight(t.text:GetHeight()+60)
			t:UpdateBackdrop()

			
			self.parent.subframes[t] = t
			t.parentframe = self.parent
			t:SetFocus(true)
			for _,f in pairs(Frames) do
				if not(f == t) then
					if f.SetFocus then
						f:SetFocus(nil)
					end
				end 
			end 
		old = body
	end
	self:SetText(old)
end

function EditBox.GetText(self)
	return self.body:GetText()
end

function EditBox.SetFormatted(self,bool)
	self.format = bool
	self.body.format = bool
end

function EditBox.WrapParent(self, bool)
	self.wrapparent = true
end

function EditBox.SetColor(self,color)
	self.body:SetColor(color)
end

--===================== Button  =========================
--======================================================

local Button = {} --comes with a fontstring by default (also needs a texture)
Button.__index = Frame
setmetatable(Button, Button)

function Button.Create(parent, font, size)

	local temp = CreateFrame("Frame", parent)
	temp.__index = Button
	setmetatable(temp, temp)
	
	temp.NormalText = temp:CreateFontString("DEJAVUSANSMONO.TTF",10)
	temp.NormalText:SetPoint(5,2)
	
	return temp
end

function Button.SetText(self, text)
	self.NormalText:SetText(text)
end

local Blip = {}
Blip.__index = Frame
setmetatable(Blip, Blip)

function Blip.Create(parent, color)

	local temp = CreateFrame("Frame", parent)
	temp.__index = Blip
	setmetatable(temp, temp)
	
	temp:SetSize(8,8)
	
	temp.tex = temp:CreateTexture()
	temp.tex:SetSize(8,8)
	temp.tex:SetTexture("Textures/blip.png")
	temp.tex:SetColor(color)
	temp.color = color
	
	
	return temp
end


--======================== EditFrame =======================
--==========================================================

--[[
edit frame combines a fonstring title with an editbox and blips to control itself
--]]

local EditFrame = {} --contains a title text and editbox
EditFrame.__index = Frame
setmetatable(EditFrame, EditFrame)

function EditFrame.Create(parent)

	local temp = Frame.Create(parent)
	temp.__index = EditFrame
	setmetatable(temp, temp)
	
	temp:SetSize(200, 200)
	--temp:SetScript("OnEnter", function() print("cheese") end)
	temp:SetBackdrop("Textures/template.png", 32)
	temp.mouseEnabled = true
	temp.subframes = {}
	
	temp:SetScript("OnMouseDown", function(self,x,y,button)
																	self:StartMoving() 
																end)
	temp:SetScript("OnMouseUp", function(self) self:StopMovingOrSizing() end)
	
	temp.title = temp:CreateFontString("DEJAVUSANSMONO.TTF", 10)
	
	temp.title:SetPoint(28,20)
	temp.title:SetText("Title")
	temp.title:SetColor({33,176,36,255})
	
	
	temp.text = EditBox.Create(temp, "DEJAVUSANSMONO.TTF", 10)
	temp.text:SetPoint(25,40)
	temp.text:SetText("")
	temp.text:WrapParent(true)
	temp.text:SetFormatted(true)
	
	temp:SetScript("OnKeyDown", function(self, key, uni)
		
																if self.focus then 
																	self.text:EditText(key,uni)													
																else
																	self.text:SetText(self.text:GetText())
																end
															end)

	
	temp.collapse = CreateFrame("Blip", temp,{43,214,54,255})
	temp.collapse:SetPoint(24, 6)
	temp.collapse:SetScript("OnMouseDown", function(self)
																					if temp.text.isshown then
																							temp.text:Hide()
																							self.tex:SetColor({120,120,120,255})
																									if self.parent.subframes then
																										for f in pairs(self.parent.subframes) do
																											f.dx = f.x - self.parent:GetWidth()
																											f.dy = f.y - self.parent:GetHeight()
																										end
																									end

																							temp:SetHeight(40)
																							temp:UpdateBackdrop()
																							if self.parent.subframes then
																								for f in pairs(self.parent.subframes) do
																									f.x = self.parent:GetWidth() + f.dx
																									f.y = self.parent:GetHeight() + f.dy
																								end
																							end
																						else
																							temp.text:Show()
																							temp.text:SetText(temp.text:GetText())
																							self.tex:SetColor(self.color)
																					end
																				end)
																				
	temp.collapseall = CreateFrame("Blip", temp,{41,187,227,255})
	temp.collapseall:SetPoint(32,6)
	
	temp.collapseall:SetScript("OnMouseDown", function(self)
																					if temp.collapsed then
																							for f in pairs(temp.subframes) do
																								f.isshown = true
																							end
																							temp.collapsed = false
																							self.tex:SetColor(self.color)
																							temp:UpdateBackdrop()
																						else
																							for f in pairs(temp.subframes) do
																								f.isshown = false
																							end
																							temp.collapsed = true
																							self.tex:SetColor({120,120,120,255})
																							temp:UpdateBackdrop()
																					end
																				end)
																				
	temp.showlinks = CreateFrame("Blip", temp,{201,60,60,255})
	temp.showlinks:SetPoint(40,6)
	
	temp.showlinks:SetScript("OnMouseDown", function(self)
																					if temp.hidelinks then
																						self.tex:SetColor(self.color)
																						temp.hidelinks = false
																					else
																						self.tex:SetColor({120,120,120,255})
																						temp.hidelinks = true
																					end
																				end)
	

	
	return temp
end



function EditFrame.SetTitle(self,title)
	self.title:SetText(title)
	self:SetMinWidth(self.title.font:getWidth(self.title.text)+100)
	self:SetWidth(self.title.font:getWidth(self.title.text)+60)
end

function EditFrame.GetTitle(self)
	return self.title:GetText()
end

function EditFrame.GetText(self)
	return self.text:GetText()
	
end

function EditFrame.SetText(self, text)
	self.text:SetText(text)
end


--===================== Project ===========================
--=========================================================

local function NewFile(self) --adds a new file to the current project																		
	local temp = CreateFrame("EditFrame", self.parent)
	local title = "New"
	
	for _,f in pairs(self.parent.children) do
		if f.GetTitle then
		if f:GetTitle():find("New") then
			title = "New"..((f:GetTitle():match("New(%d)\.lua")or 0) + 1)
		end
		end
	end
	temp:SetTitle(title..".lua")
	temp:SetText(temp:GetText())
	local t = self.parent.children
	t = t[#t]
	self.parent.main:SetText('require "'..title..'"\n'..self.parent.main:GetText())
	temp.file = true
	temp:SetPoint(t:GetX()-self.parent:GetX() + t:GetWidth() + 40, t:GetY())
	self.tex:SetColor(self.color)
end

local function SaveProj(self)
	local msg = "--============= GLOBALS ===============\n"
	msg = msg..self.parent.globals:GetText()
	msg = msg.."\n--======================================="
	
	msg = msg..self.parent.main:GetText()
	
	local title = self.parent.title:GetText()
	
	if not love.filesystem.exists(title) then
		love.filesystem.mkdir("Projects/"..title)
	end
	
	
	love.filesystem.write("Projects/"..title.."/main.lua", msg)
	
	self.tex:SetColor(self.color)

end

local Project = {}
Project.__index = Frame
setmetatable(Project, Project)

--[[
project:
contains file names
global variables
local variables
--]]

function Project.Create(parent)

	local temp = CreateFrame("Frame", parent)
	temp.__index = Project
	setmetatable(temp, temp)
	if #Projects > 0 then
		temp:SetPoint(Projects[#Projects]:GetX()+200,0) 
	else
		temp:SetPoint(200, 0)
	end
	
	temp:SetSize(160, 70)
	temp:SetBackdrop("Textures/template.png", 32)
	temp.file = true
	
	--temp.close = CreateFrame("Blip", temp,{255,20,20,255})
	--temp.close:SetPoint(100, 6)
	--temp.close:SetScript("OnMouseDown", function(self) self.parent:Hide() end)
	
	temp.newfile = CreateFrame("Blip", temp, {20, 240, 20, 255})
	temp.newfile:SetPoint(120, 6)
	temp.newfile:SetScript("OnMouseDown", function(self) self.tex:SetColor({120,120,120}) end)
	temp.newfile:SetScript("OnMouseUp", NewFile)
	
	temp.saveproj = CreateFrame("Blip", temp, {20, 180, 20, 255})
	temp.saveproj:SetPoint(90, 6)
	temp.saveproj:SetScript("OnMouseDown", function(self) self.tex:SetColor({120,120,120}) end)
	temp.saveproj:SetScript("OnMouseUp", SaveProj)
	
	
	temp:SetScript("OnMouseDown", function(self,x,y,button)
																	self:StartMoving() 
																end)
	temp:SetScript("OnMouseUp", function(self) self:StopMovingOrSizing() end)
	
	temp.title = EditBox.Create(temp, "DEJAVUSANSMONO.TTF", 12)
	temp.title:SetColor({50,50,220,255})
	temp.title:SetPoint(28,25)
	temp.title:SetText("New Project")
	temp.title:WrapParent(true)
	
	temp:SetScript("OnKeyDown", function(self, key, uni)
		
																if self.focus then 
																	--self.text:EditText(key,uni)	
																	if not (uni == 13) then
																	
																		temp.title:EditText(key,uni)
																		temp:SetHeight(50)
																		temp:UpdateBackdrop()
																	end
																else
																	--self.text:SetText(self.text:GetText())
																	
																end
															end)



	temp.globals = EditFrame.Create(temp)
	temp.globals:SetPoint(140, 0)
	temp.globals:SetTitle("Globals")
	temp.globals:SetSize(160, 20)
	temp.globals:UpdateBackdrop()
	temp.globals.file = true
	
	temp.main = EditFrame.Create(temp)
	temp.main:SetPoint(0, 80)
	temp.main:SetTitle("main.lua")
	temp.main:SetSize(160, 20)
	temp.main:UpdateBackdrop()
	
	temp.main:SetText(MAIN_DEFAULT_TEXT)
	temp.main.file = true
	tinsert(Projects, temp)
	return temp
end




function CreateFrame(typ, parent,...)
	if parent and parent.children then
		if typ == "Frame" then
			return Frame.Create(parent)
		elseif typ == "Button" then
			return Button.Create(parent)
		elseif typ == "EditFrame" then
			return EditFrame.Create(parent)
		elseif typ == "EditBox" then
			return EditBox.Create(parent)
		elseif typ == "Project" then
			return Project.Create(parent)
		elseif typ == "Blip" then
			return Blip.Create(parent,...)
		end
	end
end



