


UIParent = {}

Frames = {} --flat list of all frames

Strings = {} -- all strings

Projects = {}

Scripts = {}
Scripts.OnKeyDown = {}
Scripts.OnMouseDown = {}
Scripts.OnEnter = {}
Scripts.OnLeave = {}
Scripts.OnMouseUp = {}
Scripts.OnUpdate = {}

UIParent.children = {}


function UIParent.GetWidth(self)
	return self.width
end

function UIParent.GetHeight(self)
	return self.height
end

function UIParent.GetX(self)
	return self.x
end

function UIParent.GetY(self)
	return self.y
end

function UIParent.GetRot(self)
	return self.rot
end

function UIParent.GetPoint(self)
	return self.x, self.y
end

function UIParent.SetPoint(self,x,y)
	self.x = x
	self.y = y
end

function UIParent.IsVisible(self)
	return true
end

function UIParent.SetScale(self,x,y)
	self.scalex = x
	self.scaley = y
end

function UIParent.GetScale(self)
	return self.scalex, self.scaley
end

function UIParent.SetScaleX(self,x)
	self.scalex = x
end

function UIParent.GetScaleX(self)
	return self.scalex
end

function UIParent.SetScaleY(self,y)
	self.scaley = y
end

function UIParent.GetScaleY(self)
	return self.scaley
end

UIParent.width = love.graphics.getWidth()
UIParent.height = love.graphics.getHeight()
UIParent.x = 0
UIParent.y = 0
UIParent.rot = 0
UIParent.scalex = 1
UIParent.scaley = 1
UIParent.vx = 0
UIParent.vy = 0

local Settings = {}

require "Globals"
require "Widgets"
require "Options"


local tinsert = table.insert

tinsert(Frames, UIParent)



local t = CreateFrame("Project", UIParent)

t:SetPoint(100,100)





function UIParent.draw(self)
	for i, child in pairs(self.children) do
		if child.isshown then
			child:draw()
		end
	end
end

function UIParent.StartMoving(self)
	local mx, my = love.mouse.getPosition()
	self.xoff = mx - self.x
	self.yoff = my - self.y
	self.moving = true
end

function UIParent.StopMovingOrSizing(self)
	self.moving = false
	self.sizing = false 
end

function UIParent.CheckMouse(self, x, y)
	--if x > self:GetX() and x < self:GetX()+self:GetWidth() and y > self:GetY() and y < self:GetY()+self:GetHeight() then
	for _, f in pairs(Frames) do
		if f.mouseEnabled and f.isshown then
			f:CheckMouse(x,y)
		end
	end
end

function UIParent.MouseDown(self,x,y,button)
	local found 
	
	for _,child in pairs(Frames) do
		if child.mouseEnabled and child.mouseover then
			found = true
			child:MouseDown(x,y,button)
		end
	end
	
	if not found then
		if button == "wu" then
			self:SetScale(self:GetScaleX()+0.05, self:GetScaleY()+0.05)
			local mx, my = love.mouse.getPosition()
			local sx, sy = self:GetPoint()
			self:SetPoint(mx - (mx-sx)*1.05, my - (my-sy)*1.05)
			for _,f in pairs(Frames) do
				if f.backdrop then
					f:UpdateBackdrop()
				end
			end
		elseif button == "wd" then
			self:SetScale(self:GetScaleX()-0.05, self:GetScaleY()-0.05)
			local mx, my = love.mouse.getPosition()
			local sx, sy = self:GetPoint()
			self:SetPoint(mx - (mx-sx)*0.95, my - (my-sy)*0.95)
			for _,f in pairs(Frames) do
				if f.backdrop then
					f:UpdateBackdrop()
				end
			end
		else
			self:StartMoving()
			for _,f in pairs(Frames) do 
				if f.SetFocus then
					
					f:SetFocus(nil)
				end
			end
		end
	end
end

function UIParent.MouseUp(self, x,y,button)
	local found

	for _,child in pairs(Frames) do
			if child.mouseEnabled and child.mouseover then
				found = true
				child:MouseUp(x,y,button)
			end
		end
	if not found then	
		if self.moving then
			self:StopMovingOrSizing()
		end
	end
end

local function DrawLines()
	for _,f in pairs(Frames) do
		if f:IsVisible()and not f.hidelinks then
			if f.subframes then
				for subf in pairs(f.subframes) do
					if subf:IsVisible() then
						local cfx, cfy = f:GetX()+(f:GetWidth())/2, f:GetY() + (f:GetHeight())/2
						local cnx, cny = subf:GetX()+(subf:GetWidth())/2, subf:GetY() + (subf:GetHeight())/2
						
						local theta = math.atan2((cnx-cfx),(cny-cfy))
						local x,y = (f:GetWidth()/2)*math.sin(theta), (f:GetHeight()/2)*math.cos(theta)
						local cx,cy =  -1*(subf:GetWidth()/2)*math.sin(theta), -1*(subf:GetHeight()/2)*math.cos(theta)
								
						love.graphics.setColor(255,100,100,150)
						love.graphics.line( cfx+x, cfy+y, cx+cnx, cy+cny)
					end
				end
			end
		end
	end
end

--=========== LOAD ======================

function love.load()
	love.graphics.setBackgroundColor(20,20,70,255)
end

--=============== KEYBOARD ==============

function love.keypressed(key,uni)
	Strings = {}
	for child, script in pairs(Scripts.OnKeyDown) do
		script(child,key,uni)
	end
end

local mx, my = 0,0

--================= UPDATE =====================

function love.update(dt)

	mx, my = love.mouse.getPosition()
	UIParent:CheckMouse(mx,my)
	
	
	for f, func in pairs(Scripts.OnUpdate) do
		func(f, dt)
	end
	
	for _,child in pairs(Frames) do
		if child.moving then
			child.x = mx - child.xoff
			child.y = my - child.yoff
		end
	end
end

-- ================== MOUSE =================

function love.mousepressed(x,y,button)
	UIParent:MouseDown(x,y,button)
end

function love.mousereleased(x,y,button)
	UIParent:MouseUp(x,y,button)
end

-- ================= DRAW =====================

function love.draw()
	UIParent:draw()	
	DrawLines()
	love.graphics.print(love.timer.getFPS(), 400, 400)

end